"""inkhinge包的核心功能模块"""
import os
import numpy as np
import pandas as pd
from spectrochempy_omnic import OMNICReader as read
from decimal import Decimal, Context, ROUND_HALF_UP
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.signal import find_peaks

"read_to_csv模块："
def read_to_csv(input_path, output_path=None, background_path=None, overwrite=False, recursive=False, precision=20, merge_output=None):
    """
    读取Omnic SPA文件并转换为CSV格式，可选择性合并所有转换后的CSV文件

    参数:
        input_path (str): 输入SPA文件路径或包含这些文件的目录路径
        output_path (str): 输出CSV文件路径或目录路径
        background_path (str): 背景BG.spa文件路径
        overwrite (bool): 是否覆盖已存在的文件
        recursive (bool): 是否递归处理子目录(仅在处理目录时有效)
        precision (int): 输出数据的小数位数精度
        merge_output (str): 合并所有CSV文件的输出路径，默认为None表示不合并

    返回:
        处理结果信息(成功转换的文件数量或单个文件的输出路径)
    """
    def float_to_fixed_str(value, precision=20):
        """将浮点数转换为指定精度的定点小数表示字符串"""
        if np.isnan(value):
            return 'nan'
        try:
            ctx = Context(prec=max(25, precision + 5), rounding=ROUND_HALF_UP)
            dec = ctx.create_decimal(str(value))
            return f"{dec:.{precision}f}"
        except:
            return str(value)

    def detect_data_type(reader):
        """检测数据类型并返回相应的标题和单位"""
        data_type_mapping = {
            0: ("Absorbance", "AU"),
            1: ("Transmittance", "%"),
            2: ("Reflectance", "%"),
            3: ("Single Beam", ""),
            4: ("Kubelka-Munk", "KM units"),
        }

        if hasattr(reader, 'data_type') and reader.data_type in data_type_mapping:
            return data_type_mapping[reader.data_type]

        if hasattr(reader, 'title'):
            title = reader.title.lower()
            if "absorbance" in title:
                return "Absorbance", "AU"
            elif "transmittance" in title or "透过率" in title:
                return "Transmittance", "%"
            elif "reflectance" in title:
                return "Reflectance", "%"
            elif "single beam" in title or "单光束" in title:
                return "Single Beam", ""
            elif "kubelka-munk" in title or "km" in title:
                return "Kubelka-Munk", "KM units"

        y_title = reader.y_title or "Intensity"
        y_units = reader.y_units or ""

        if "kubelka" in y_title.lower() or "km" in y_title.lower():
            return "Kubelka-Munk", "KM units"

        return y_title, y_units

    def calculate_kubelka_munk(reflectance):
        """计算Kubelka-Munk值"""
        reflectance = np.clip(reflectance, 0.0001, 0.9999)
        return ((1 - reflectance) **2) / (2 * reflectance)

    def extract_spectral_data(reader):
        """从读取器中提取光谱数据和对应的X轴数据"""
        data = reader.data
        x = reader.x

        x_units = reader.x_units or "cm^-1"
        x_title = reader.x_title or "Wavelength"

        if data.ndim == 1:
            spectral_data = data.reshape(1, -1)
        elif data.ndim >= 2:
            spectral_dim = None

            if data.shape[-1] == len(x):
                spectral_dim = -1
            elif data.shape[0] == len(x):
                spectral_dim = 0

            if spectral_dim is None:
                for i in range(data.ndim):
                    if data.shape[i] == len(x):
                        spectral_dim = i
                        break

            if spectral_dim is None:
                spectral_dim = np.argmin(np.abs(np.array(data.shape) - len(x)))
                print(f"警告: 无法确定光谱数据维度，假设为维度 {spectral_dim}")

            if spectral_dim != -1:
                axes = list(range(data.ndim))
                axes.remove(spectral_dim)
                axes.append(spectral_dim)
                data = data.transpose(axes)

            spectral_data = data.reshape(-1, len(x))
        else:
            raise ValueError(f"不支持的数据维度: {data.ndim}")

        return spectral_data, x, x_title, x_units

    def apply_background_correction(sample_data, background_data, x_sample, x_bg):
        """应用背景校正"""
        if np.array_equal(x_sample, x_bg):
            corrected_data = sample_data / background_data
        else:
            corrected_data = np.zeros_like(sample_data)
            for i, spectrum in enumerate(sample_data):
                bg_interp = np.interp(x_sample, x_bg, background_data[0])
                corrected_data[i] = spectrum / bg_interp

        return corrected_data

    def convert_spa_to_csv(input_file, output_file=None, background_path=None, overwrite=False, precision=20):
        """将Omnic SPA文件转换为CSV格式"""
        if not os.path.exists(input_file):
            raise FileNotFoundError(f"输入文件不存在: {input_file}")

        if not output_file:
            base_name, _ = os.path.splitext(input_file)
            output_file = f"{base_name}_converted.csv"

        if os.path.exists(output_file) and not overwrite:
            raise FileExistsError(f"输出文件已存在: {output_file}")

        try:
            print(f"正在读取样本文件: {input_file}")
            sample_reader = read(input_file)

            sample_data, x_sample, x_title, x_units = extract_spectral_data(sample_reader)
            y_title, y_units = detect_data_type(sample_reader)

            if background_path:
                if not os.path.exists(background_path):
                    raise FileNotFoundError(f"背景文件不存在: {background_path}")

                print(f"正在读取背景文件: {background_path}")
                bg_reader = read(background_path)
                bg_data, x_bg, _, _ = extract_spectral_data(bg_reader)

                if y_title == "Reflectance":
                    corrected_data = apply_background_correction(sample_data, bg_data, x_sample, x_bg)
                    y_title = "Corrected Reflectance"
                elif y_title == "Transmittance":
                    corrected_data = sample_data - bg_data
                    y_title = "Corrected Transmittance"
                else:
                    corrected_data = apply_background_correction(sample_data, bg_data, x_sample, x_bg)
                    y_title = f"Corrected {y_title}"

                spectral_data = corrected_data
            else:
                spectral_data = sample_data

            print(f"数据维度: {spectral_data.shape}")
            print(f"X轴: {x_title} ({x_units})")
            print(f"数据类型: {y_title} ({y_units})")

            # 创建DataFrame列数据（解决碎片化问题）
            columns_data = {
                f"{x_title} ({x_units})": [float_to_fixed_str(val, precision) for val in x_sample]
            }

            if y_title == "Reflectance" or y_title == "Corrected Reflectance":
                km_data = calculate_kubelka_munk(spectral_data)
                km_title = "Kubelka-Munk"
                km_units = "KM units"

                if km_data.shape[0] == 1:
                    columns_data[f"{km_title} ({km_units})"] = [float_to_fixed_str(val, precision) for val in km_data[0]]
                else:
                    for i in range(km_data.shape[0]):
                        columns_data[f"{km_title}_{i} ({km_units})"] = [float_to_fixed_str(val, precision) for val in km_data[i]]

            if spectral_data.shape[0] == 1:
                columns_data[f"{y_title} ({y_units})"] = [float_to_fixed_str(val, precision) for val in spectral_data[0]]
            else:
                if hasattr(sample_reader, 'spectra_titles') and len(sample_reader.spectra_titles) == spectral_data.shape[0]:
                    for i, title in enumerate(sample_reader.spectra_titles):
                        clean_title = title.strip() or f"{y_title}_{i}"
                        columns_data[f"{clean_title} ({y_units})"] = [float_to_fixed_str(val, precision) for val in spectral_data[i]]
                else:
                    for i in range(spectral_data.shape[0]):
                        columns_data[f"{y_title}_{i} ({y_units})"] = [float_to_fixed_str(val, precision) for val in spectral_data[i]]

            df = pd.DataFrame(columns_data)
            df.to_csv(output_file, index=False, na_rep='nan')
            print(f"成功转换并保存至: {output_file}")
            return output_file

        except Exception as e:
            print(f"转换失败: {str(e)}")
            return None

    def batch_convert_spa_to_csv(input_dir, output_dir=None, background_path=None, overwrite=False, recursive=False,
                                 precision=20, merge_output=None):
        """批量转换目录中的SPA文件为CSV格式，并可选择性合并所有CSV文件"""
        if not os.path.exists(input_dir):
            raise FileNotFoundError(f"输入目录不存在: {input_dir}")

        if output_dir and not os.path.exists(output_dir):
            os.makedirs(output_dir)

        spa_files = []
        for root, _, files in os.walk(input_dir):
            for file in files:
                if file.lower().endswith('.spa'):
                    spa_files.append(os.path.join(root, file))

            if not recursive:
                break

        if not spa_files:
            print(f"在目录 {input_dir} 中未找到SPA文件")
            return []

        # 按文件名排序
        spa_files.sort()
        output_files = []
        for spa_file in spa_files:
            try:
                if output_dir:
                    rel_path = os.path.relpath(spa_file, input_dir)
                    base_name, _ = os.path.splitext(rel_path)
                    output_file = os.path.join(output_dir, f"{base_name}.csv")
                    os.makedirs(os.path.dirname(output_file), exist_ok=True)
                else:
                    output_file = None

                result = convert_spa_to_csv(spa_file, output_file, background_path, overwrite, precision)
                if result:
                    output_files.append(result)
            except Exception as e:
                print(f"处理文件 {spa_file} 时出错: {str(e)}")

        print(f"批量转换完成: 成功 {len(output_files)}/{len(spa_files)}")

        # 合并CSV文件
        if merge_output and output_files:
            merge_csv_files(output_files, merge_output, overwrite, precision)

        return output_files

    def merge_csv_files(csv_files, output_path, overwrite=False, precision=20):
        """
        按顺序合并多个CSV文件，解决性能警告并使列名从0开始计数

        参数:
            csv_files (list): CSV文件路径列表(按顺序排列)
            output_path (str): 合并后的输出文件路径
            overwrite (bool): 是否覆盖已存在的文件
            precision (int): 输出数据的小数位数精度
        """
        if not csv_files:
            print("没有CSV文件可合并")
            return

        # 检查输出文件是否存在
        if os.path.exists(output_path) and not overwrite:
            raise FileExistsError(f"合并输出文件已存在: {output_path}")

        print(f"开始合并 {len(csv_files)} 个CSV文件...")

        # 读取所有数据框并准备合并
        data_frames = []
        x_column = None

        for i, csv_file in enumerate(csv_files):
            df = pd.read_csv(csv_file)

            # 验证X轴列
            current_x_col = df.columns[0]
            if i == 0:
                x_column = current_x_col
                # 保留第一个文件的X轴列，并重命名数据列添加后缀_0
                rename_map = {col: f"{col}_0" for col in df.columns[1:]}
                renamed_df = df.rename(columns=rename_map)
                data_frames.append(renamed_df)
            else:
                if current_x_col != x_column:
                    print(f"警告: 文件 {csv_file} 的X轴列名与第一个文件不一致: {current_x_col} vs {x_column}")
                    continue

                # 重命名后续文件的数据列，从1开始计数
                file_suffix = f"_{i}"
                rename_map = {col: f"{col}{file_suffix}" for col in df.columns[1:]}
                renamed_df = df.rename(columns=rename_map)
                # 移除后续文件的X轴列
                data_frames.append(renamed_df.drop(columns=[current_x_col]))

            print(f"已准备文件 {i+1}/{len(csv_files)}: {csv_file}")

        # 一次性合并所有数据框（解决性能警告）
        merged_df = pd.concat(data_frames, axis=1)

        # 使用自定义格式化函数保存CSV，确保定点小数精度
        float_format = lambda x: float_to_fixed_str(x, precision)
        merged_df.to_csv(output_path, index=False, na_rep='nan', float_format=float_format)

        print(f"成功合并并保存至: {output_path}")
        return output_path

    # 判断输入是文件还是目录
    if os.path.isfile(input_path):
        # 确保输入是.spa文件
        if not input_path.lower().endswith('.spa'):
            raise ValueError(f"输入文件不是SPA文件: {input_path}")
        # 处理单个文件
        result = convert_spa_to_csv(input_path, output_path, background_path, overwrite, precision)
        if merge_output:
            return [result] if result else []
        return result
    elif os.path.isdir(input_path):
        # 处理目录
        return batch_convert_spa_to_csv(input_path, output_path, background_path, overwrite, recursive, precision, merge_output)
    else:
        raise ValueError(f"输入路径不存在: {input_path}")

"curvefit_km_t模块："
def curvefit_km_t(file_path, target_row=1, txt_output_path="curvefit_km_t.txt", png_output_path="curvefit_km_t.png", show_plot=True):
    """
    读取CSV文件，对指定行的KM units数据进行曲线拟合，并可视化结果

    参数:
    file_path (str): CSV文件路径
    target_row (int, optional): 目标行号(从1开始)，默认为1
    txt_output_path (str, optional): 输出拟合结果的文本文件路径，默认为当前目录下的curvefit_km_t.txt
    png_output_path (str, optional): 输出拟合图像的PNG文件路径，默认为当前目录下的curvefit_km_t.png
    show_plot (bool, optional): 是否显示图表，默认为True

    返回:
    dict: 包含拟合参数、评估指标和原始数据的字典
    """
    try:
        # 读取CSV文件
        df = pd.read_csv(file_path)
    except Exception as e:
        print(f"读取文件出错: {e}")
        return None

    # 检查数据行数
    rows, columns = df.shape
    if rows < target_row:
        print(f'数据行数为{rows}，少于指定行{target_row}，无法获取对应行数据')
        return None

    # 获取指定行数据
    row_data = df.iloc[target_row - 1]
    title = row_data[0]

    # 提取时间和KM values
    time = range(1, len(row_data))
    km_values = row_data[1:].values

    # 设置图片清晰度和中文字体
    plt.rcParams['figure.dpi'] = 150
    plt.rcParams['font.sans-serif'] = ['WenQuanYi Zen Hei', 'SimHei', 'Heiti TC']

    # 定义拟合函数: y = a * exp(-b * x) + c
    def exponential_func(x, a, b, c):
        return a * np.exp(-b * x) + c

    # 初始参数猜测
    a_guess = max(km_values) - min(km_values)
    b_guess = 0.1
    c_guess = min(km_values)
    p0 = [a_guess, b_guess, c_guess]

    try:
        # 执行曲线拟合
        popt, pcov = curve_fit(exponential_func, time, km_values, p0=p0)

        # 计算拟合值和R²
        y_fit = exponential_func(time, *popt)
        residuals = km_values - y_fit
        ss_res = np.sum(residuals ** 2)
        ss_tot = np.sum((km_values - np.mean(km_values)) ** 2)
        r_squared = 1 - (ss_res / ss_tot)

        # 生成拟合函数表达式
        fit_expr = f"y = {popt[0]:.6f} * exp(-{popt[1]:.6f} * x) + {popt[2]:.6f}"

        # 写入拟合结果到文本文件
        with open(txt_output_path, "w") as f:
            f.write(f"拟合结果 - {title}\n\n")
            f.write("拟合函数表达式:\n")
            f.write(f"{fit_expr}\n\n")
            f.write("拟合参数:\n")
            f.write(f"a = {popt[0]:.6f}\n")
            f.write(f"b = {popt[1]:.6f}\n")
            f.write(f"c = {popt[2]:.6f}\n\n")
            f.write(f"R² = {r_squared:.6f}\n")

        print(f"拟合结果已成功写入 {txt_output_path}")

        # 创建图表
        plt.figure(figsize=(7, 5))
        plt.scatter(time, km_values, marker='o', label='原始数据')
        plt.plot(time, y_fit, 'r-',
                 label=f'拟合曲线: {fit_expr}\nR² = {r_squared:.4f}')

        # 设置图表属性
        plt.xlabel('T (s)')
        plt.ylabel('KM units')
        plt.title(title)
        plt.legend()
        plt.grid(True, linestyle='--', alpha=0.7)
        plt.tight_layout()

        # 保存图像
        plt.savefig(png_output_path, dpi=300, bbox_inches='tight')
        print(f"拟合图像已成功保存至 {png_output_path}")

        # 显示图表（如果需要）
        if show_plot:
            plt.show()

        # 返回结果字典
        return {
            'title': title,
            'time': np.array(time),
            'km_values': km_values,
            'parameters': {'a': popt[0], 'b': popt[1], 'c': popt[2]},
            'r_squared': r_squared,
            'fitted_values': y_fit,
            'fit_expression': fit_expr
        }

    except Exception as e:
        print(f"拟合过程出错: {e}")
        return None

"curvefit_km_wavenumber模块："
def curvefit_km_wavenumber(file_path, time_column_index=1, txt_output_path="curvefit_km_wavenumber.txt",
                      png_output_path="curvefit_km_wavenumber.png"):
    """
    拟合特定时间下KM值与波数的高斯-洛伦兹混合函数

    参数:
        file_path: CSV文件路径，每行代表一个波数，每列代表某个时间下的各波数的KM值
        time_column_index: 需要拟合的时间列索引（从0开始），默认第1列
        txt_output_path: 输出文本文件路径，默认当前目录下的curvefit_km_wavenumber.txt
        png_output_path: 输出图像文件路径，默认当前目录下的curvefit_km_wavenumber.png

    返回:
        包含拟合结果的字典，包括拟合参数、R²值和函数表达式
    """

    # 高斯-洛伦兹混合函数（3个峰）
    def gaussian_lorentzian(x, a1, x01, sigma1, eta1, a2, x02, sigma2, eta2, a3, x03, sigma3, eta3):
        g1 = a1 * np.exp(-(x - x01) ** 2 / (2 * sigma1 ** 2))
        l1 = a1 / (1 + ((x - x01) / sigma1) ** 2)
        g2 = a2 * np.exp(-(x - x02) ** 2 / (2 * sigma2 ** 2))
        l2 = a2 / (1 + ((x - x02) / sigma2) ** 2)
        g3 = a3 * np.exp(-(x - x03) ** 2 / (2 * sigma3 ** 2))
        l3 = a3 / (1 + ((x - x03) / sigma3) ** 2)
        return eta1 * l1 + (1 - eta1) * g1 + eta2 * l2 + (1 - eta2) * g2 + eta3 * l3 + (1 - eta3) * g3

    # 计算R²值（拟合优度）
    def calculate_r_squared(actual, predicted):
        ss_res = np.sum((actual - predicted) ** 2)
        ss_tot = np.sum((actual - np.mean(actual)) ** 2)
        return 1 - (ss_res / ss_tot) if ss_tot != 0 else 0

    # 1. 数据读取与预处理
    df = pd.read_csv(file_path)

    wavenumbers = df.iloc[:, 0].values  # 波数列为第0列

    # 确保波数单调递增
    is_increasing = np.all(np.diff(wavenumbers) >= 0)
    if not is_increasing:
        print("检测到波数非单调递增，自动排序修正...")
        sorted_indices = np.argsort(wavenumbers)
        wavenumbers = wavenumbers[sorted_indices]
        df = df.iloc[sorted_indices]

    # 筛选1300-1320 cm⁻¹范围
    mask = (wavenumbers >= 1300) & (wavenumbers <= 1320)
    valid_wavenumbers = wavenumbers[mask]
    km_values = df.iloc[mask, time_column_index].values  # 获取指定时间列的KM值

    # 2. 三峰拟合
    def fit_3peaks(x, y):
        x_min, x_max = x.min(), x.max()
        x_range = x_max - x_min
        y_max = y.max()

        # 增强峰检测
        peaks, _ = find_peaks(y, height=np.percentile(y, 70), distance=5)
        if len(peaks) < 3:
            peaks = [
                np.argmax(y),
                int(len(x) * 0.25),
                int(len(x) * 0.75)
            ]
        peaks = sorted(peaks)

        # 物理约束初始值
        p0 = [
            y[peaks[0]] * 0.9, x[peaks[0]], max(x_range * 0.05, 0.05), 0.3,
            y[peaks[1]] * 0.6, x[peaks[1]], max(x_range * 0.08, 0.08), 0.5,
            y[peaks[2]] * 0.4, x[peaks[2]], max(x_range * 0.06, 0.06), 0.7
        ]

        # 合理约束
        bounds = (
            [0, x_min, 0.01, 0.2, 0, x_min, 0.01, 0.2, 0, x_min, 0.01, 0.2],
            [np.inf, x_max, x_range, 0.8, np.inf, x_max, x_range, 0.8, np.inf, x_max, x_range, 0.8]
        )

        # 确保初始值在约束范围内
        for i in range(len(p0)):
            p0[i] = max(bounds[0][i], min(p0[i], bounds[1][i]))

        try:
            popt, pcov = curve_fit(
                gaussian_lorentzian, x, y, p0=p0, bounds=bounds,
                maxfev=100000, method='trf'
            )
            predicted = gaussian_lorentzian(x, *popt)
            r_squared = calculate_r_squared(y, predicted)
            return popt, r_squared
        except Exception as e:
            print(f"三峰拟合警告: {e}，启用备用策略")
            # 备用初始值
            simplified_popt = [
                y_max * 0.5, x.mean() - x_range * 0.2, x_range * 0.05, 0.4,
                y_max * 0.3, x.mean(), x_range * 0.07, 0.5,
                y_max * 0.2, x.mean() + x_range * 0.2, x_range * 0.06, 0.6
            ]
            # 确保备用初始值在约束内
            for i in range(len(simplified_popt)):
                simplified_popt[i] = max(bounds[0][i], min(simplified_popt[i], bounds[1][i]))
            popt, pcov = curve_fit(
                gaussian_lorentzian, x, y, p0=simplified_popt,
                bounds=bounds, maxfev=100000, method='trf'
            )
            predicted = gaussian_lorentzian(x, *popt)
            r_squared = calculate_r_squared(y, predicted)
            return popt, r_squared

    # 拟合KM与波数的关系
    popt, r2 = fit_3peaks(valid_wavenumbers, km_values)

    # 3. 生成拟合函数表达式
    def format_gaussian_lorentzian(params):
        a1, x01, sigma1, eta1, a2, x02, sigma2, eta2, a3, x03, sigma3, eta3 = params
        expr = "KM(w) = "
        expr += f"{eta1:.6f} * ({a1:.6f} / (1 + ((w - {x01:.6f}) / {sigma1:.6f})^2)) + "
        expr += f"{1 - eta1:.6f} * ({a1:.6f} * exp(-(w - {x01:.6f})^2 / (2 * {sigma1:.6f}^2))) + "
        expr += f"{eta2:.6f} * ({a2:.6f} / (1 + ((w - {x02:.6f}) / {sigma2:.6f})^2)) + "
        expr += f"{1 - eta2:.6f} * ({a2:.6f} * exp(-(w - {x02:.6f})^2 / (2 * {sigma2:.6f}^2))) + "
        expr += f"{eta3:.6f} * ({a3:.6f} / (1 + ((w - {x03:.6f}) / {sigma3:.6f})^2)) + "
        expr += f"{1 - eta3:.6f} * ({a3:.6f} * exp(-(w - {x03:.6f})^2 / (2 * {sigma3:.6f}^2)))"
        return expr

    # 生成函数表达式
    km_expr = format_gaussian_lorentzian(popt)

    # 写入文件
    with open(txt_output_path, "w") as f:
        f.write("===== 拟合函数表达式 =====\n\n")
        f.write("KM与波数的高斯-洛伦兹混合函数:\n")
        f.write(f"{km_expr}\n\n")
        f.write("===== 拟合优度 =====\n")
        f.write(f"R²值: {r2:.6f}\n")

    print(f"拟合函数表达式已成功写入 {txt_output_path} 文件")

    # 4. 可视化拟合结果
    plt.rcParams["font.family"] = ["Arial Unicode MS", "Heiti TC", "sans-serif"]
    plt.rcParams["axes.unicode_minus"] = True

    plt.figure(figsize=(12, 6))
    plt.scatter(valid_wavenumbers, km_values, color='blue', label='原始数据')

    # 生成拟合曲线
    x_fit = np.linspace(valid_wavenumbers.min(), valid_wavenumbers.max(), 1000)
    y_fit = gaussian_lorentzian(x_fit, *popt)
    plt.plot(x_fit, y_fit, 'r-', label=f'拟合曲线 (R²={r2:.4f})')

    plt.title('KM值与波数的拟合关系')
    plt.xlabel('波数 (cm⁻¹)')
    plt.ylabel('KM值')
    plt.legend()
    plt.grid(True)
    plt.tight_layout()

    # 保存图像
    plt.savefig(png_output_path, dpi=300, bbox_inches='tight')
    print(f"拟合结果图像已成功保存至 {png_output_path}")

    # 显示图像（可选）
    plt.show()

    # 返回关键拟合结果
    return {
        'popt': popt,
        'r2': r2,
        'km_expr': km_expr,
        'valid_wavenumbers': valid_wavenumbers,
        'km_values': km_values
    }


"curvefit_km_t_wavenumber模块："
def curvefit_km_t_wavenumber(file_path, wavenumber_min=1300, wavenumber_max=1320,
                             output_txt_path="curvefit.txt", output_img_path="curvefit.png"):
    """
    对时间-波数-KM值数据进行三维拟合，返回拟合结果并生成可视化和文本报告

    参数:
        file_path (str): 数据CSV文件路径
        wavenumber_min (float): 波数范围下限，默认为1300 cm⁻¹
        wavenumber_max (float): 波数范围上限，默认为1320 cm⁻¹
        output_txt_path (str): 输出文本报告路径，默认为当前目录下的"curvefit.txt"
        output_img_path (str): 输出图像路径，默认为当前目录下的"curvefit.png"

    返回:
        dict: 包含所有拟合结果的字典，包括拟合参数、R²值等
    """

    # 高斯-洛伦兹混合函数（3个峰）
    def gaussian_lorentzian(x, a1, x01, sigma1, eta1, a2, x02, sigma2, eta2, a3, x03, sigma3, eta3):
        g1 = a1 * np.exp(-(x - x01) ** 2 / (2 * sigma1 ** 2))
        l1 = a1 / (1 + ((x - x01) / sigma1) ** 2)
        g2 = a2 * np.exp(-(x - x02) ** 2 / (2 * sigma2 ** 2))
        l2 = a2 / (1 + ((x - x02) / sigma2) ** 2)
        g3 = a3 * np.exp(-(x - x03) ** 2 / (2 * sigma3 ** 2))
        l3 = a3 / (1 + ((x - x03) / sigma3) ** 2)
        return eta1 * l1 + (1 - eta1) * g1 + eta2 * l2 + (1 - eta2) * g2 + eta3 * l3 + (1 - eta3) * g3

    # 原始指数时间函数（用于初步获取b值计算k）
    def exp_func_original(t, a, b, c):
        return a * np.exp(-b * t) + c

    # 计算R²值（拟合优度）
    def calculate_r_squared(actual, predicted):
        ss_res = np.sum((actual - predicted) ** 2)
        ss_tot = np.sum((actual - np.mean(actual)) ** 2)
        return 1 - (ss_res / ss_tot) if ss_tot != 0 else 0

    # 1. 数据读取与预处理（核心：解决坐标轴反转）
    df = pd.read_csv(file_path)

    # 获取第一列作为波数数据
    wavenumbers = df.iloc[:, 0].values
    first_column_name = df.columns[0]
    print(f"已自动获取第一列 '{first_column_name}' 作为波数数据")

    # 关键优化1：强制波数单调递增（解决坐标轴反转的根本原因）
    is_increasing = np.all(np.diff(wavenumbers) >= 0)  # 波数是否从左到右递增
    if not is_increasing:
        print("检测到波数非单调递增，自动排序修正以解决坐标轴反转...")
        sorted_indices = np.argsort(wavenumbers)  # 按波数升序排序的索引
        wavenumbers = wavenumbers[sorted_indices]  # 波数按升序排列（1300→1320）
        df = df.iloc[sorted_indices]  # 数据行同步排序（确保波数与数据对应）

    # 筛选指定波数范围
    mask = (wavenumbers >= wavenumber_min) & (wavenumbers <= wavenumber_max)
    valid_wavenumbers = wavenumbers[mask]
    time = np.arange(1, df.shape[1])  # 时间序列（1,2,...N）

    print(f"使用波数范围: {wavenumber_min} - {wavenumber_max} cm⁻¹")
    print(f"筛选得到 {len(valid_wavenumbers)} 个有效波数数据点")

    # 2. 分两步拟合：先获取b值计算常数k，再固定k拟合a(w)和c(w)
    # 第一步：初步拟合获取所有波数的b值以计算常数k
    a_temp, b_temp, c_temp = [], [], []
    for w in valid_wavenumbers:
        try:
            row_idx = np.where(wavenumbers == w)[0][0]
            km_values = df.iloc[row_idx, 1:].values  # 提取该波数下的时间序列数据

            # 优化初始值：基于数据首尾趋势
            early_mean = np.mean(km_values[:int(len(km_values) * 0.1)])
            late_mean = np.mean(km_values[-int(len(km_values) * 0.1):])
            a_guess = early_mean - late_mean
            a_guess = max(a_guess, 0.1) if a_guess <= 0 else a_guess  # 保证a为正

            # 动态调整b初始值
            b_guess = 0.05 if km_values[-1] > km_values[0] * 0.5 else 0.2
            c_guess = late_mean

            # 约束拟合
            popt, _ = curve_fit(
                exp_func_original, time, km_values, p0=[a_guess, b_guess, c_guess],
                maxfev=20000, bounds=([0, 0, 0], [np.inf, np.inf, np.inf])
            )
            a_temp.append(popt[0])
            b_temp.append(popt[1])
            c_temp.append(popt[2])
        except Exception as e:
            print(f"波数 {w} 初步拟合失败: {e}")
            a_temp.append(np.nan)
            b_temp.append(np.nan)
            c_temp.append(np.nan)

    # 计算常数k（取所有有效b值的平均值）
    k = np.nanmean(b_temp)
    print(f"计算得到的常数k值为: {k:.6f}")

    # 过滤无效数据（基于初步拟合结果）
    valid_mask = ~np.isnan(a_temp) & ~np.isnan(b_temp) & ~np.isnan(c_temp)
    valid_wavenumbers = valid_wavenumbers[valid_mask]
    a_temp = np.array(a_temp)[valid_mask]
    b_temp = np.array(b_temp)[valid_mask]
    c_temp = np.array(c_temp)[valid_mask]

    if len(valid_wavenumbers) < 12:
        raise ValueError("有效数据点不足（至少需要12个）")

    # 第二步：固定k值，重新拟合每个波数的a(w)和c(w)
    def exp_func_fixed_k(t, a, c):
        """固定k后的指数时间函数，仅拟合a和c"""
        return a * np.exp(-k * t) + c

    a_list, c_list = [], []
    a_r2_list, c_r2_list = [], []

    for i, w in enumerate(valid_wavenumbers):
        try:
            row_idx = np.where(wavenumbers == w)[0][0]
            km_values = df.iloc[row_idx, 1:].values  # 提取该波数下的时间序列数据

            # 基于初步拟合结果优化初始值
            a_guess = a_temp[i]
            c_guess = c_temp[i]

            # 约束拟合（仅拟合a和c）
            popt, _ = curve_fit(
                exp_func_fixed_k, time, km_values, p0=[a_guess, c_guess],
                maxfev=20000, bounds=([0, 0], [np.inf, np.inf])  # 约束a和c为非负
            )
            predicted = exp_func_fixed_k(time, *popt)
            r_squared = calculate_r_squared(km_values, predicted)

            a_list.append(popt[0])
            c_list.append(popt[1])
            a_r2_list.append(r_squared)
            c_r2_list.append(r_squared)
        except Exception as e:
            print(f"波数 {w} 二次拟合失败: {e}")
            a_list.append(np.nan)
            c_list.append(np.nan)
            a_r2_list.append(np.nan)
            c_r2_list.append(np.nan)

    # 再次过滤二次拟合中的无效数据
    valid_mask2 = ~np.isnan(a_list) & ~np.isnan(c_list)
    valid_wavenumbers = valid_wavenumbers[valid_mask2]
    a_vals = np.array(a_list)[valid_mask2]
    c_vals = np.array(c_list)[valid_mask2]
    a_r2 = np.array(a_r2_list)[valid_mask2]
    c_r2 = np.array(c_r2_list)[valid_mask2]

    if len(valid_wavenumbers) < 12:
        raise ValueError("二次拟合后有效数据点不足（至少需要12个）")

    # 3. 三峰拟合（仅拟合a和c的光谱分布）
    def fit_3peaks(x, y):
        x_min, x_max = x.min(), x.max()
        x_range = x_max - x_min
        y_max = y.max()

        # 增强峰检测：优先检测合理峰位
        peaks, _ = find_peaks(y, height=np.percentile(y, 70), distance=5)
        if len(peaks) < 3:
            peaks = [
                np.argmax(y),  # 全局最高峰
                int(len(x) * 0.25),  # 左次峰
                int(len(x) * 0.75)  # 右次峰
            ]
        peaks = sorted(peaks)  # 保证左-中-右顺序

        # 物理约束初始值：确保在约束范围内
        p0 = [
            y[peaks[0]] * 0.9, x[peaks[0]], max(x_range * 0.05, 0.05), 0.3,
            y[peaks[1]] * 0.6, x[peaks[1]], max(x_range * 0.08, 0.08), 0.5,
            y[peaks[2]] * 0.4, x[peaks[2]], max(x_range * 0.06, 0.06), 0.7
        ]

        # 合理约束：限制参数范围
        bounds = (
            [0, x_min, 0.01, 0.2, 0, x_min, 0.01, 0.2, 0, x_min, 0.01, 0.2],
            [np.inf, x_max, x_range, 0.8, np.inf, x_max, x_range, 0.8, np.inf, x_max, x_range, 0.8]
        )

        # 关键优化：确保初始值在约束范围内
        for i in range(len(p0)):
            p0[i] = max(bounds[0][i], min(p0[i], bounds[1][i]))

        try:
            # 使用TRF算法（支持约束）
            popt, pcov = curve_fit(
                gaussian_lorentzian, x, y, p0=p0, bounds=bounds,
                maxfev=100000, method='trf'
            )
            predicted = gaussian_lorentzian(x, *popt)
            r_squared = calculate_r_squared(y, predicted)
            return popt, r_squared
        except Exception as e:
            print(f"三峰拟合警告: {e}，启用备用策略")
            # 备用初始值（更保守）
            simplified_popt = [
                y_max * 0.5, x.mean() - x_range * 0.2, x_range * 0.05, 0.4,
                y_max * 0.3, x.mean(), x_range * 0.07, 0.5,
                y_max * 0.2, x.mean() + x_range * 0.2, x_range * 0.06, 0.6
            ]
            # 确保备用初始值在约束内
            for i in range(len(simplified_popt)):
                simplified_popt[i] = max(bounds[0][i], min(simplified_popt[i], bounds[1][i]))
            popt, pcov = curve_fit(
                gaussian_lorentzian, x, y, p0=simplified_popt,
                bounds=bounds, maxfev=100000, method='trf'
            )
            predicted = gaussian_lorentzian(x, *popt)
            r_squared = calculate_r_squared(y, predicted)
            return popt, r_squared

    # 拟合a和c的光谱分布
    popt_a, r2_a = fit_3peaks(valid_wavenumbers, a_vals)
    popt_c, r2_c = fit_3peaks(valid_wavenumbers, c_vals)

    # 4. 三维KM函数构建（使用常数k）
    def km_3d(t, w):
        a = gaussian_lorentzian(w, *popt_a)
        c = gaussian_lorentzian(w, *popt_c)
        return a * np.exp(-k * t) + c

    # 5. 生成网格数据（确保坐标轴顺序正确）
    t_grid = np.linspace(time[0], time[-1], 30)  # 时间网格
    w_grid = valid_wavenumbers.reshape(-1, 1)  # 波数网格（升序）

    # 计算拟合曲面
    km_grid = np.zeros((len(valid_wavenumbers), len(t_grid)))
    for i, w in enumerate(valid_wavenumbers):
        for j, t in enumerate(t_grid):
            km_grid[i, j] = km_3d(t, w)

    # 构建原始数据网格（严格对齐）
    original_km_grid = np.zeros_like(km_grid)
    for i, w in enumerate(valid_wavenumbers):
        row_idx = np.where(wavenumbers == w)[0][0]
        km_values = df.iloc[row_idx, 1:].values  # 原始时间序列数据
        original_km_grid[i, :] = np.interp(t_grid, time, km_values)  # 插值对齐

    # 6. 计算整体R²（逐点对齐）
    actual_values = original_km_grid.flatten()
    predicted_values = km_grid.flatten()
    r2_overall = calculate_r_squared(actual_values, predicted_values)

    # 7. 将拟合函数表达式写入文件
    def format_gaussian_lorentzian(params, func_name):
        """格式化高斯-洛伦兹混合函数表达式"""
        a1, x01, sigma1, eta1, a2, x02, sigma2, eta2, a3, x03, sigma3, eta3 = params
        expr = f"{func_name}(w) = "
        expr += f"{eta1:.6f} * ({a1:.6f} / (1 + ((w - {x01:.6f}) / {sigma1:.6f})^2)) + "
        expr += f"{1 - eta1:.6f} * ({a1:.6f} * exp(-(w - {x01:.6f})^2 / (2 * {sigma1:.6f}^2))) + "
        expr += f"{eta2:.6f} * ({a2:.6f} / (1 + ((w - {x02:.6f}) / {sigma2:.6f})^2)) + "
        expr += f"{1 - eta2:.6f} * ({a2:.6f} * exp(-(w - {x02:.6f})^2 / (2 * {sigma2:.6f}^2))) + "
        expr += f"{eta3:.6f} * ({a3:.6f} / (1 + ((w - {x03:.6f}) / {sigma3:.6f})^2)) + "
        expr += f"{1 - eta3:.6f} * ({a3:.6f} * exp(-(w - {x03:.6f})^2 / (2 * {sigma3:.6f}^2)))"
        return expr

    # 生成函数表达式
    a_expr = format_gaussian_lorentzian(popt_a, "a")
    c_expr = format_gaussian_lorentzian(popt_c, "c")
    km_3d_expr = f"km_3d(t, w) = {a_expr.split('=')[1].strip()} * exp(-{k:.6f} * t) + {c_expr.split('=')[1].strip()}"

    # 确保输出目录存在
    output_txt_dir = os.path.dirname(output_txt_path)
    output_img_dir = os.path.dirname(output_img_path)

    if output_txt_dir and not os.path.exists(output_txt_dir):
        os.makedirs(output_txt_dir)
        print(f"创建目录: {output_txt_dir}")

    if output_img_dir and not os.path.exists(output_img_dir):
        os.makedirs(output_img_dir)
        print(f"创建目录: {output_img_dir}")

    # 写入文件
    with open(output_txt_path, "w") as f:
        f.write("===== 拟合函数表达式 =====\n\n")
        f.write("1. 参数a的高斯-洛伦兹混合函数:\n")
        f.write(f"{a_expr}\n\n")
        f.write("2. 参数c的高斯-洛伦兹混合函数:\n")
        f.write(f"{c_expr}\n\n")
        f.write("3. 三维KM函数:\n")
        f.write(f"{km_3d_expr}\n\n")
        f.write(f"使用波数范围: {wavenumber_min} - {wavenumber_max} cm⁻¹\n\n")
        f.write(f"CSV第一列标题: '{first_column_name}'\n\n")
        f.write("===== 拟合优度 =====\n")
        f.write(f"参数a的R²值: {r2_a:.6f}\n")
        f.write(f"参数c的R²值: {r2_c:.6f}\n")
        f.write(f"常数k值: {k:.6f}\n")
        f.write(f"整体曲面R²值: {r2_overall:.6f}\n")
        f.write(f"时间函数平均R²值: {np.mean(a_r2):.6f}\n")

    print(f"拟合函数表达式已成功写入 {output_txt_path}")

    # 8. 可视化对比
    plt.rcParams["font.family"] = ["Arial Unicode MS", "Heiti TC", "sans-serif"]
    plt.rcParams["axes.unicode_minus"] = True

    fig = plt.figure(figsize=(18, 8))

    # 原始数据曲面
    ax1 = fig.add_subplot(121, projection='3d')
    surf1 = ax1.plot_surface(
        t_grid, w_grid, original_km_grid,
        cmap='viridis', alpha=0.9, edgecolor='none'
    )
    ax1.set_xlabel('时间 t (s)')
    ax1.set_ylabel(f'波数 ({wavenumber_min}-{wavenumber_max} cm⁻¹)')
    ax1.set_zlabel('KM值')
    ax1.set_title('原始数据')

    # 拟合数据曲面
    ax2 = fig.add_subplot(122, projection='3d')
    surf2 = ax2.plot_surface(
        t_grid, w_grid, km_grid,
        cmap='plasma', alpha=0.9, edgecolor='none'
    )
    ax2.set_xlabel('时间 t (s)')
    ax2.set_ylabel(f'波数 ({wavenumber_min}-{wavenumber_max} cm⁻¹)')
    ax2.set_zlabel('KM值')
    ax2.set_title(f'拟合曲面 (整体R²: {r2_overall:.4f})')

    # 同步视角
    ax2.view_init(elev=30, azim=45)
    ax1.view_init(elev=30, azim=45)

    # 拟合评估信息
    stats_text = (f"拟合评估:\n"
                  f"• 参数 a 的R²: {r2_a:.4f}\n"
                  f"• 参数 c 的R²: {r2_c:.4f}\n"
                  f"• 常数k值: {k:.6f}\n"
                  f"• 时间函数平均R²: {np.mean(a_r2):.4f}\n"
                  f"• 整体曲面R²: {r2_overall:.4f}\n"
                  f"• 波数范围: {wavenumber_min}-{wavenumber_max} cm⁻¹\n"
                  f"• CSV第一列: '{first_column_name}'")
    fig.text(0.01, 0.01, stats_text, fontsize=9, bbox=dict(facecolor='white', alpha=0.8))

    # 颜色条
    cbar_ax = fig.add_axes([0.92, 0.15, 0.02, 0.7])
    fig.colorbar(surf2, cax=cbar_ax)

    plt.tight_layout()
    plt.subplots_adjust(right=0.9)

    # 保存图像
    plt.savefig(output_img_path, dpi=300, bbox_inches='tight')
    print(f"可视化图像已成功保存至 {output_img_path}")

    # 显示图像
    plt.show()

    # 输出评估结果
    print(f"===== 拟合评估结果 =====")
    print(f"CSV第一列标题: '{first_column_name}'")
    print(f"使用波数范围: {wavenumber_min} - {wavenumber_max} cm⁻¹")
    print(f"参数 a 的R²: {r2_a:.4f}")
    print(f"参数 c 的R²: {r2_c:.4f}")
    print(f"常数k值: {k:.6f}")
    print(f"时间函数平均R²: {np.mean(a_r2):.4f}")
    print(f"整体曲面R²: {r2_overall:.4f}")

    # 返回拟合结果
    return {
        'k': k,
        'popt_a': popt_a,
        'popt_c': popt_c,
        'r2_a': r2_a,
        'r2_c': r2_c,
        'r2_overall': r2_overall,
        'mean_time_r2': np.mean(a_r2),
        'km_3d_func': km_3d,
        'valid_wavenumbers': valid_wavenumbers,
        'time': time,
        'wavenumber_min': wavenumber_min,
        'wavenumber_max': wavenumber_max,
        'first_column_name': first_column_name,
        'output_txt_path': output_txt_path,
        'output_img_path': output_img_path
    }

